﻿using System.Windows.Interactivity;
using System.Windows;
using System.Windows.Input;
using System;
using Infragistics.Windows.Ribbon;
using Infragistics.Windows.Ribbon.Events;
using CaliburnMicroWPFTest.Framework;
using System.Collections.Generic;
namespace CaliburnMicroWPFTest
{
   public class MethodTrigger : TriggerBase<FrameworkElement>, ICommand
   {
      private bool attachedOnLoad = false;
      private KeyBinding InputBinding = new KeyBinding();

      public event EventHandler CanExecuteChanged = delegate { };

      public ModifierKeys Modifier { get; set; }

      public Key Key { get; set; }

      public string AttachedEvents { get; set; }

      public string MethodName { get; set; }


      public bool CanExecute(object parameter)
      {
         return true;
      }

      /// <summary>
      /// Executes the method associated with the keyboard shortcut.       
      /// </summary>
      /// <param name="parameter"></param>
      public void Execute(object parameter)
      {
         InvokeActions(parameter);

         var window = GetWindow(AssociatedObject);
         ExecuteMethodWithGuardCheck(window.DataContext, MethodName);
      }

      /// <summary>
      /// Executes the method from the data context checking first if a guard is available and allows the execution. 
      /// REMARK: The method is executed using the Caliburn.Micro Framework, but the Guard execution is implemented in this method. No guard check implementation could be found in the Caliburn.Micro.
      /// </summary>
      public static bool ExecuteMethodWithGuardCheck(object dataContext, string methodName)
      {
         var guardMethod = dataContext.GetType().GetMethod("Can" + methodName);
         var guardProperty = dataContext.GetType().GetProperty("Can" + methodName);
         var canExecute = true;

         try
         {
            if (guardMethod != null)
            {
               canExecute = (bool)guardMethod.Invoke(dataContext, new object[] { });
            }

            if (guardProperty != null)
            {
               canExecute = (bool)guardProperty.GetValue(dataContext, null);
            }
         }
         catch
         {
            Console.WriteLine("InputBindingTrigger: Guard read Error.");
            return false;
         }

         if (canExecute)
         {
            CaliburnMicroWPFTest.Framework.Action.Invoke(dataContext, methodName);
            return true;
         }

         return false;
      }

      protected override void OnAttached()
      {
         if (AttachedEvents != string.Empty)
         {
            AssociatedObject.Loaded += (sender, e) =>
            {
               if (!attachedOnLoad)
               {
                  Message.SetAttach(sender as DependencyObject, AttachedEvents);
               }
               attachedOnLoad = true;
            };
         }

         if (MethodName != string.Empty)
         {
            var window = GetWindow(AssociatedObject);

            InputBinding.Modifiers = Modifier;
            InputBinding.Key = Key;
            InputBinding.Command = this;

            AssociatedObject.Loaded += (sender, e) =>
            {
               window.InputBindings.Add(InputBinding);
            };
         }

         base.OnAttached();
      }
      
      private Window GetWindow(FrameworkElement frameworkElement)
      {
         if (frameworkElement is Window)
         {
            return frameworkElement as Window;
         }

         var parent = frameworkElement.Parent as FrameworkElement;         
         return GetWindow(parent);
      }

   } 

}
